@@ -148,8 +148,10 @@ struct librados::IoCtxImpl {
   struct C_aio_sparse_read_Ack : public Context {
     AioCompletionImpl *c;
     bufferlist *data_bl;
-    std::map<uint64_t,uint64_t> *m;
-    C_aio_sparse_read_Ack(AioCompletionImpl *_c);
+    std::map<uint64_t, uint64_t> *m;
+    C_aio_sparse_read_Ack(AioCompletionImpl *_c,
+			  bufferlist *data,
+			  std::map<uint64_t, uint64_t> *extents);
     void finish(int r);
   };
 
@@ -172,6 +174,7 @@ struct librados::IoCtxImpl {
 		 const bufferlist& bl, size_t len);
   int aio_write_full(const object_t &oid, AioCompletionImpl *c,
 		     const bufferlist& bl);
+  int aio_remove(const object_t &oid, AioCompletionImpl *c);
   int aio_exec(const object_t& oid, AioCompletionImpl *c, const char *cls,
 	       const char *method, bufferlist& inbl, bufferlist *outbl);
 
@@ -200,11 +203,9 @@ struct librados::IoCtxImpl {
 };
 
 namespace librados {
-struct WatchContext {
+struct WatchContext : public RefCountedWaitObject {
   IoCtxImpl *io_ctx_impl;
   const object_t oid;
-  uint64_t cookie;
-  uint64_t ver;
   librados::WatchCtx *ctx;
   uint64_t linger_id;
 
@@ -212,7 +213,7 @@ struct WatchContext {
 	       const object_t& _oc,
 	       librados::WatchCtx *_ctx);
   ~WatchContext();
-  void notify(uint8_t opcode, uint64_t ver, uint64_t notify_id,
+  void notify(Mutex *lock, uint8_t opcode, uint64_t ver, uint64_t notify_id,
 	      bufferlist& payload);
 };
 }
