@@ -28,30 +28,34 @@ using librados::IoCtx;
 
 namespace librbd {
   ImageCtx::ImageCtx(const string &image_name, const string &image_id,
-		     const char *snap, IoCtx& p)
+		     const char *snap, IoCtx& p, bool ro)
     : cct((CephContext*)p.cct()),
       perfcounter(NULL),
       snap_id(CEPH_NOSNAP),
       snap_exists(true),
+      read_only(ro),
       exclusive_locked(false),
       name(image_name),
       wctx(NULL),
       refresh_seq(0),
       last_refresh(0),
-      last_header_version(0),
       md_lock("librbd::ImageCtx::md_lock"),
       cache_lock("librbd::ImageCtx::cache_lock"),
       snap_lock("librbd::ImageCtx::snap_lock"),
       parent_lock("librbd::ImageCtx::parent_lock"),
       refresh_lock("librbd::ImageCtx::refresh_lock"),
       old_format(true),
-      order(0), size(0), features(0),	id(image_id), parent(NULL),
+      order(0), size(0), features(0),
+      format_string(NULL),
+      id(image_id), parent(NULL),
+      stripe_unit(0), stripe_count(0),
       object_cacher(NULL), writeback_handler(NULL), object_set(NULL)
   {
     md_ctx.dup(p);
     data_ctx.dup(p);
 
     memset(&header, 0, sizeof(header));
+    memset(&layout, 0, sizeof(layout));
 
     string pname = string("librbd-") + id + string("-") +
       data_ctx.get_pool_name() + string("/") + name;
@@ -69,10 +73,12 @@ namespace librbd {
       object_cacher = new ObjectCacher(cct, pname, *writeback_handler, cache_lock,
 				       NULL, NULL,
 				       cct->_conf->rbd_cache_size,
+				       10,  /* reset this in init */
 				       cct->_conf->rbd_cache_max_dirty,
 				       cct->_conf->rbd_cache_target_dirty,
 				       cct->_conf->rbd_cache_max_dirty_age);
       object_set = new ObjectCacher::ObjectSet(NULL, data_ctx.get_id(), 0);
+      object_set->return_enoent = true;
       object_cacher->start();
     }
   }
@@ -91,6 +97,7 @@ namespace librbd {
       delete object_set;
       object_set = NULL;
     }
+    delete[] format_string;
   }
 
   int ImageCtx::init() {
@@ -123,35 +130,82 @@ namespace librbd {
 		   << cpp_strerror(r) << dendl;
 	return r;
       }
+
+      r = cls_client::get_stripe_unit_count(&md_ctx, header_oid,
+					    &stripe_unit, &stripe_count);
+      if (r < 0 && r != -ENOEXEC && r != -EINVAL) {
+	lderr(cct) << "error reading striping metadata: "
+		   << cpp_strerror(r) << dendl;
+	return r;
+      }
+
+      init_layout();
     } else {
       header_oid = old_header_name(name);
     }
-
     return 0;
   }
+  
+  void ImageCtx::init_layout()
+  {
+    if (stripe_unit == 0 || stripe_count == 0) {
+      stripe_unit = 1ull << order;
+      stripe_count = 1;
+    }
+
+    memset(&layout, 0, sizeof(layout));
+    layout.fl_stripe_unit = stripe_unit;
+    layout.fl_stripe_count = stripe_count;
+    layout.fl_object_size = 1ull << order;
+    layout.fl_pg_pool = data_ctx.get_id();  // FIXME: pool id overflow?
+
+    delete[] format_string;
+    size_t len = object_prefix.length() + 16;
+    format_string = new char[len];
+    if (old_format) {
+      snprintf(format_string, len, "%s.%%012llx", object_prefix.c_str());
+    } else {
+      snprintf(format_string, len, "%s.%%016llx", object_prefix.c_str());
+    }
+
+    // size object cache appropriately
+    if (object_cacher) {
+      uint64_t obj = cct->_conf->rbd_cache_size / (1ull << order);
+      ldout(cct, 10) << " cache bytes " << cct->_conf->rbd_cache_size << " order " << (int)order
+		     << " -> about " << obj << " objects" << dendl;
+      object_cacher->set_max_objects(obj * 4 + 10);
+    }
+
+    ldout(cct, 10) << "init_layout stripe_unit " << stripe_unit
+		   << " stripe_count " << stripe_count
+		   << " object_size " << layout.fl_object_size
+		   << " prefix " << object_prefix
+		   << " format " << format_string
+		   << dendl;
+  }
 
   void ImageCtx::perf_start(string name) {
     PerfCountersBuilder plb(cct, name, l_librbd_first, l_librbd_last);
 
     plb.add_u64_counter(l_librbd_rd, "rd");
     plb.add_u64_counter(l_librbd_rd_bytes, "rd_bytes");
-    plb.add_fl_avg(l_librbd_rd_latency, "rd_latency");
+    plb.add_time_avg(l_librbd_rd_latency, "rd_latency");
     plb.add_u64_counter(l_librbd_wr, "wr");
     plb.add_u64_counter(l_librbd_wr_bytes, "wr_bytes");
-    plb.add_fl_avg(l_librbd_wr_latency, "wr_latency");
+    plb.add_time_avg(l_librbd_wr_latency, "wr_latency");
     plb.add_u64_counter(l_librbd_discard, "discard");
     plb.add_u64_counter(l_librbd_discard_bytes, "discard_bytes");
-    plb.add_fl_avg(l_librbd_discard_latency, "discard_latency");
+    plb.add_time_avg(l_librbd_discard_latency, "discard_latency");
     plb.add_u64_counter(l_librbd_flush, "flush");
     plb.add_u64_counter(l_librbd_aio_rd, "aio_rd");
     plb.add_u64_counter(l_librbd_aio_rd_bytes, "aio_rd_bytes");
-    plb.add_fl_avg(l_librbd_aio_rd_latency, "aio_rd_latency");
+    plb.add_time_avg(l_librbd_aio_rd_latency, "aio_rd_latency");
     plb.add_u64_counter(l_librbd_aio_wr, "aio_wr");
     plb.add_u64_counter(l_librbd_aio_wr_bytes, "aio_wr_bytes");
-    plb.add_fl_avg(l_librbd_aio_wr_latency, "aio_wr_latency");
+    plb.add_time_avg(l_librbd_aio_wr_latency, "aio_wr_latency");
     plb.add_u64_counter(l_librbd_aio_discard, "aio_discard");
     plb.add_u64_counter(l_librbd_aio_discard_bytes, "aio_discard_bytes");
-    plb.add_fl_avg(l_librbd_aio_discard_latency, "aio_discard_latency");
+    plb.add_time_avg(l_librbd_aio_discard_latency, "aio_discard_latency");
     plb.add_u64_counter(l_librbd_snap_create, "snap_create");
     plb.add_u64_counter(l_librbd_snap_remove, "snap_remove");
     plb.add_u64_counter(l_librbd_snap_rollback, "snap_rollback");
@@ -228,13 +282,64 @@ namespace librbd {
     return -ENOENT;
   }
 
+  uint64_t ImageCtx::get_current_size() const
+  {
+    return size;
+  }
+
+  uint64_t ImageCtx::get_object_size() const
+  {
+    return 1ull << order;
+  }
+
+  string ImageCtx::get_object_name(uint64_t num) const {
+    char buf[object_prefix.length() + 32];
+    snprintf(buf, sizeof(buf), format_string, num);
+    return string(buf);
+  }
+
+  uint64_t ImageCtx::get_stripe_unit() const
+  {
+    return stripe_unit;
+  }
+
+  uint64_t ImageCtx::get_stripe_count() const
+  {
+    return stripe_count;
+  }
+
+  uint64_t ImageCtx::get_stripe_period() const
+  {
+    return stripe_count * (1ull << order);
+  }
+
+  uint64_t ImageCtx::get_num_objects() const
+  {
+    uint64_t period = get_stripe_period();
+    uint64_t num_periods = (size + period - 1) / period;
+    return num_periods * stripe_count;
+  }
+
   int ImageCtx::is_snap_protected(string in_snap_name, bool *is_protected) const
   {
     assert(snap_lock.is_locked());
     map<string, SnapInfo>::const_iterator it = snaps_by_name.find(in_snap_name);
     if (it != snaps_by_name.end()) {
       *is_protected =
-	(it->second.protection_status != RBD_PROTECTION_STATUS_UNPROTECTED);
+	(it->second.protection_status == RBD_PROTECTION_STATUS_PROTECTED);
+      return 0;
+    }
+    return -ENOENT;
+  }
+
+  int ImageCtx::is_snap_unprotected(string in_snap_name,
+				    bool *is_unprotected) const
+  {
+    assert(snap_lock.is_locked());
+    map<string, SnapInfo>::const_iterator it = snaps_by_name.find(in_snap_name);
+    if (it != snaps_by_name.end()) {
+      *is_unprotected =
+	(it->second.protection_status == RBD_PROTECTION_STATUS_UNPROTECTED);
       return 0;
     }
     return -ENOENT;
@@ -373,14 +478,14 @@ namespace librbd {
     snap_lock.Lock();
     ObjectCacher::OSDRead *rd = object_cacher->prepare_read(snap_id, bl, 0);
     snap_lock.Unlock();
-    ObjectExtent extent(o, off, len);
+    ObjectExtent extent(o, 0 /* a lie */, off, len);
     extent.oloc.pool = data_ctx.get_id();
-    extent.buffer_extents[0] = len;
+    extent.buffer_extents.push_back(make_pair(0, len));
     rd->extents.push_back(extent);
     cache_lock.Lock();
     int r = object_cacher->readx(rd, object_set, onfinish);
     cache_lock.Unlock();
-    if (r > 0)
+    if (r != 0)
       onfinish->complete(r);
   }
 
@@ -390,9 +495,9 @@ namespace librbd {
     ObjectCacher::OSDWrite *wr = object_cacher->prepare_write(snapc, bl,
 							      utime_t(), 0);
     snap_lock.Unlock();
-    ObjectExtent extent(o, off, len);
+    ObjectExtent extent(o, 0, off, len);
     extent.oloc.pool = data_ctx.get_id();
-    extent.buffer_extents[0] = len;
+    extent.buffer_extents.push_back(make_pair(0, len));
     wr->extents.push_back(extent);
     {
       Mutex::Locker l(cache_lock);
@@ -463,20 +568,7 @@ namespace librbd {
   int ImageCtx::register_watch() {
     assert(!wctx);
     wctx = new WatchCtx(this);
-
-    int r = 0;
-    do {
-      md_lock.Lock();
-      if (r == -ERANGE)
-	ictx_refresh(this);
-      uint64_t last_read = last_header_version;
-      md_lock.Unlock();
-
-      md_ctx.set_assert_version(last_read);
-      r = md_ctx.watch(header_oid, 0, &(wctx->cookie), wctx);
-      ldout(cct, 20) << "watching header object returned " << r << dendl;
-    } while (r == -ERANGE);
-    return r;
+    return md_ctx.watch(header_oid, 0, &(wctx->cookie), wctx);
   }
 
   void ImageCtx::unregister_watch() {
@@ -504,4 +596,26 @@ namespace librbd {
 		   << parent_len << dendl;
     return parent_len;
   }
+
+  uint64_t ImageCtx::prune_parent_extents(vector<pair<uint64_t,uint64_t> >& objectx,
+					  uint64_t overlap)
+  {
+    // drop extents completely beyond the overlap
+    while (!objectx.empty() && objectx.back().first >= overlap)
+      objectx.pop_back();
+
+    // trim final overlapping extent
+    if (!objectx.empty() && objectx.back().first + objectx.back().second > overlap)
+      objectx.back().second = overlap - objectx.back().first;
+
+    uint64_t len = 0;
+    for (vector<pair<uint64_t,uint64_t> >::iterator p = objectx.begin();
+	 p != objectx.end();
+	 ++p)
+      len += p->second;
+    ldout(cct, 10) << "prune_parent_extents image overlap " << overlap
+		   << ", object overlap " << len
+		   << " from image extents " << objectx << dendl;
+    return len;
+ }
 }
