@@ -37,7 +37,7 @@
  * Accepter
  */
 
-int Accepter::bind(entity_addr_t &bind_addr, int avoid_port1, int avoid_port2)
+int Accepter::bind(const entity_addr_t &bind_addr, int avoid_port1, int avoid_port2)
 {
   const md_config_t *conf = msgr->cct->_conf;
   // bind to a socket
@@ -59,10 +59,8 @@ int Accepter::bind(entity_addr_t &bind_addr, int avoid_port1, int avoid_port2)
   listen_sd = ::socket(family, SOCK_STREAM, 0);
   if (listen_sd < 0) {
     char buf[80];
-    ldout(msgr->cct,0) << "accepter.bind unable to create socket: "
-	    << strerror_r(errno, buf, sizeof(buf)) << dendl;
-    cerr << "accepter.bind unable to create socket: "
-	 << strerror_r(errno, buf, sizeof(buf)) << std::endl;
+    lderr(msgr->cct) << "accepter.bind unable to create socket: "
+		     << strerror_r(errno, buf, sizeof(buf)) << dendl;
     return -errno;
   }
 
@@ -87,15 +85,13 @@ int Accepter::bind(entity_addr_t &bind_addr, int avoid_port1, int avoid_port2)
     rc = ::bind(listen_sd, (struct sockaddr *) &listen_addr.ss_addr(), listen_addr.addr_size());
     if (rc < 0) {
       char buf[80];
-      ldout(msgr->cct,0) << "accepter.bind unable to bind to " << bind_addr.ss_addr()
-	      << ": " << strerror_r(errno, buf, sizeof(buf)) << dendl;
-      cerr << "accepter.bind unable to bind to " << bind_addr.ss_addr()
-	   << ": " << strerror_r(errno, buf, sizeof(buf)) << std::endl;
+      lderr(msgr->cct) << "accepter.bind unable to bind to " << listen_addr.ss_addr()
+		       << ": " << strerror_r(errno, buf, sizeof(buf)) << dendl;
       return -errno;
     }
   } else {
     // try a range of ports
-    for (int port = CEPH_PORT_START; port <= CEPH_PORT_LAST; port++) {
+    for (int port = msgr->cct->_conf->ms_bind_port_min; port <= msgr->cct->_conf->ms_bind_port_max; port++) {
       if (port == avoid_port1 || port == avoid_port2)
 	continue;
       listen_addr.set_port(port);
@@ -105,12 +101,11 @@ int Accepter::bind(entity_addr_t &bind_addr, int avoid_port1, int avoid_port2)
     }
     if (rc < 0) {
       char buf[80];
-      ldout(msgr->cct,0) << "accepter.bind unable to bind to " << bind_addr.ss_addr()
-	      << " on any port in range " << CEPH_PORT_START << "-" << CEPH_PORT_LAST
-	      << ": " << strerror_r(errno, buf, sizeof(buf)) << dendl;
-      cerr << "accepter.bind unable to bind to " << bind_addr.ss_addr()
-	   << " on any port in range " << CEPH_PORT_START << "-" << CEPH_PORT_LAST
-	   << ": " << strerror_r(errno, buf, sizeof(buf)) << std::endl;
+      lderr(msgr->cct) << "accepter.bind unable to bind to " << listen_addr.ss_addr()
+		       << " on any port in range " << msgr->cct->_conf->ms_bind_port_min
+		       << "-" << msgr->cct->_conf->ms_bind_port_max
+		       << ": " << strerror_r(errno, buf, sizeof(buf))
+		       << dendl;
       return -errno;
     }
     ldout(msgr->cct,10) << "accepter.bind bound on random port " << listen_addr << dendl;
@@ -118,19 +113,22 @@ int Accepter::bind(entity_addr_t &bind_addr, int avoid_port1, int avoid_port2)
 
   // what port did we get?
   socklen_t llen = sizeof(listen_addr.ss_addr());
-  getsockname(listen_sd, (sockaddr*)&listen_addr.ss_addr(), &llen);
+  rc = getsockname(listen_sd, (sockaddr*)&listen_addr.ss_addr(), &llen);
+  if (rc < 0) {
+    rc = -errno;
+    lderr(msgr->cct) << "accepter.bind failed getsockname: " << cpp_strerror(rc) << dendl;
+    return rc;
+  }
   
   ldout(msgr->cct,10) << "accepter.bind bound to " << listen_addr << dendl;
 
   // listen!
   rc = ::listen(listen_sd, 128);
   if (rc < 0) {
-    char buf[80];
-    ldout(msgr->cct,0) << "accepter.bind unable to listen on " << listen_addr
-	    << ": " << strerror_r(errno, buf, sizeof(buf)) << dendl;
-    cerr << "accepter.bind unable to listen on " << listen_addr
-	 << ": " << strerror_r(errno, buf, sizeof(buf)) << std::endl;
-    return -errno;
+    rc = -errno;
+    lderr(msgr->cct) << "accepter.bind unable to listen on " << listen_addr
+		     << ": " << cpp_strerror(rc) << dendl;
+    return rc;
   }
   
   msgr->set_myaddr(bind_addr);
@@ -140,9 +138,11 @@ int Accepter::bind(entity_addr_t &bind_addr, int avoid_port1, int avoid_port2)
     assert(msgr->get_need_addr());  // should still be true.
 
   if (msgr->get_myaddr().get_port() == 0) {
-    listen_addr.nonce = nonce;
     msgr->set_myaddr(listen_addr);
   }
+  entity_addr_t addr = msgr->get_myaddr();
+  addr.nonce = nonce;
+  msgr->set_myaddr(addr);
 
   msgr->init_local_connection();
 
@@ -245,15 +245,19 @@ void Accepter::stop()
 {
   done = true;
   ldout(msgr->cct,10) << "stop accepter" << dendl;
-  assert(listen_sd >= 0);
-  ::shutdown(listen_sd, SHUT_RDWR);
+
+  if (listen_sd >= 0) {
+    ::shutdown(listen_sd, SHUT_RDWR);
+  }
 
   // wait for thread to stop before closing the socket, to avoid
   // racing against fd re-use.
   join();
 
-  ::close(listen_sd);
-  listen_sd = -1;
+  if (listen_sd >= 0) {
+    ::close(listen_sd);
+    listen_sd = -1;
+  }
   done = false;
 }
 
