@@ -164,6 +164,15 @@ void Message::encode(uint64_t features, bool datacrc)
       header.compat_version = header.version;
   }
   calc_front_crc();
+
+  // update envelope
+  header.front_len = get_payload().length();
+  header.middle_len = get_middle().length();
+  header.data_len = get_data().length();
+  calc_header_crc();
+
+  footer.flags = CEPH_MSG_FOOTER_COMPLETE;
+
   if (datacrc) {
     calc_data_crc();
 
@@ -196,10 +205,9 @@ void Message::encode(uint64_t features, bool datacrc)
       }
     }
 #endif
-
-  }
-  else
+  } else {
     footer.flags = (unsigned)footer.flags | CEPH_MSG_FOOTER_NOCRC;
+  }
 }
 
 void Message::dump(Formatter *f) const
