@@ -228,8 +228,8 @@ private:
       store->op_queue.pop_front();
       return osr;
     }
-    void _process(OpSequencer *osr) {
-      store->_do_op(osr);
+    void _process(OpSequencer *osr, ThreadPool::TPHandle &handle) {
+      store->_do_op(osr, handle);
     }
     void _process_finish(OpSequencer *osr) {
       store->_finish_op(osr);
@@ -239,7 +239,7 @@ private:
     }
   } op_wq;
 
-  void _do_op(OpSequencer *o);
+  void _do_op(OpSequencer *o, ThreadPool::TPHandle &handle);
   void _finish_op(OpSequencer *o);
   Op *build_op(list<Transaction*>& tls,
 	       Context *onreadable, Context *onreadable_sync,
@@ -306,9 +306,12 @@ public:
 
   int statfs(struct statfs *buf);
 
-  int do_transactions(list<Transaction*> &tls, uint64_t op_seq);
-  unsigned apply_transaction(Transaction& t, Context *ondisk=0);
-  unsigned apply_transactions(list<Transaction*>& tls, Context *ondisk=0);
+  int _do_transactions(
+    list<Transaction*> &tls, uint64_t op_seq,
+    ThreadPool::TPHandle *handle);
+  int do_transactions(list<Transaction*> &tls, uint64_t op_seq) {
+    return _do_transactions(tls, op_seq, 0);
+  }
   unsigned _do_transaction(Transaction& t, uint64_t op_seq, int trans_num);
 
   int queue_transaction(Sequencer *osr, Transaction* t);
@@ -450,6 +453,7 @@ public:
   ObjectMap::ObjectMapIterator get_omap_iterator(coll_t c, const hobject_t &hoid);
 
   int _create_collection(coll_t c);
+  int _create_collection(coll_t c, const SequencerPosition &spos);
   int _destroy_collection(coll_t c);
   int _collection_add(coll_t c, coll_t ocid, const hobject_t& o,
 		      const SequencerPosition& spos);
@@ -472,6 +476,9 @@ private:
 		      const SequencerPosition &spos);
   int _split_collection(coll_t cid, uint32_t bits, uint32_t rem, coll_t dest,
                         const SequencerPosition &spos);
+  int _split_collection_create(coll_t cid, uint32_t bits, uint32_t rem,
+			       coll_t dest,
+			       const SequencerPosition &spos);
 
   virtual const char** get_tracked_conf_keys() const;
   virtual void handle_conf_change(const struct md_config_t *conf,
