@@ -1581,6 +1581,13 @@ bool pg_interval_t::check_new_interval(
 
     if (i.acting.size() >=
 	osdmap->get_pools().find(pool_id)->second.min_size) {
+      if (out)
+	*out << "generate_past_intervals " << i
+	     << ": not rw,"
+	     << " up_thru " << lastmap->get_up_thru(i.acting[0])
+	     << " up_from " << lastmap->get_up_from(i.acting[0])
+	     << " last_epoch_clean " << last_epoch_clean
+	     << std::endl;
       if (lastmap->get_up_thru(i.acting[0]) >= i.first &&
 	  lastmap->get_up_from(i.acting[0]) <= i.first) {
 	i.maybe_went_rw = true;
@@ -1694,7 +1701,7 @@ void pg_query_t::generate_test_instances(list<pg_query_t*>& o)
 
 void pg_log_entry_t::encode(bufferlist &bl) const
 {
-  ENCODE_START(6, 4, bl);
+  ENCODE_START(7, 4, bl);
   ::encode(op, bl);
   ::encode(soid, bl);
   ::encode(version, bl);
@@ -1713,17 +1720,15 @@ void pg_log_entry_t::encode(bufferlist &bl) const
 
   ::encode(reqid, bl);
   ::encode(mtime, bl);
-  if (op == CLONE)
-    ::encode(snaps, bl);
-
   if (op == LOST_REVERT)
     ::encode(prior_version, bl);
+  ::encode(snaps, bl);
   ENCODE_FINISH(bl);
 }
 
 void pg_log_entry_t::decode(bufferlist::iterator &bl)
 {
-  DECODE_START_LEGACY_COMPAT_LEN(5, 4, 4, bl);
+  DECODE_START_LEGACY_COMPAT_LEN(7, 4, 4, bl);
   ::decode(op, bl);
   if (struct_v < 2) {
     sobject_t old_soid;
@@ -1745,8 +1750,6 @@ void pg_log_entry_t::decode(bufferlist::iterator &bl)
 
   ::decode(reqid, bl);
   ::decode(mtime, bl);
-  if (op == CLONE)
-    ::decode(snaps, bl);
   if (struct_v < 5)
     invalid_pool = true;
 
@@ -1757,6 +1760,10 @@ void pg_log_entry_t::decode(bufferlist::iterator &bl)
       reverting_to = prior_version;
     }
   }
+  if (struct_v >= 7 ||  // for v >= 7, this is for all ops.
+      op == CLONE) {    // for v < 7, it's only present for CLONE.
+    ::decode(snaps, bl);
+  }
 
   DECODE_FINISH(bl);
 }
@@ -1769,6 +1776,20 @@ void pg_log_entry_t::dump(Formatter *f) const
   f->dump_stream("prior_version") << version;
   f->dump_stream("reqid") << reqid;
   f->dump_stream("mtime") << mtime;
+  if (snaps.length() > 0) {
+    vector<snapid_t> v;
+    bufferlist c = snaps;
+    bufferlist::iterator p = c.begin();
+    try {
+      ::decode(v, p);
+    } catch (...) {
+      v.clear();
+    }
+    f->open_object_section("snaps");
+    for (vector<snapid_t>::iterator p = v.begin(); p != v.end(); ++p)
+      f->dump_unsigned("snap", *p);
+    f->close_section();
+  }
 }
 
 void pg_log_entry_t::generate_test_instances(list<pg_log_entry_t*>& o)
@@ -1781,8 +1802,20 @@ void pg_log_entry_t::generate_test_instances(list<pg_log_entry_t*>& o)
 
 ostream& operator<<(ostream& out, const pg_log_entry_t& e)
 {
-  return out << e.version << " (" << e.prior_version << ") "
-             << e.get_op_name() << ' ' << e.soid << " by " << e.reqid << " " << e.mtime;
+  out << e.version << " (" << e.prior_version << ") "
+      << e.get_op_name() << ' ' << e.soid << " by " << e.reqid << " " << e.mtime;
+  if (e.snaps.length()) {
+    vector<snapid_t> snaps;
+    bufferlist c = e.snaps;
+    bufferlist::iterator p = c.begin();
+    try {
+      ::decode(snaps, p);
+    } catch (...) {
+      snaps.clear();
+    }
+    out << " snaps " << snaps;
+  }
+  return out;
 }
 
 
@@ -2780,7 +2813,7 @@ void ScrubMap::generate_test_instances(list<ScrubMap*>& o)
 
 void ScrubMap::object::encode(bufferlist& bl) const
 {
-  ENCODE_START(4, 2, bl);
+  ENCODE_START(5, 2, bl);
   ::encode(size, bl);
   ::encode(negative, bl);
   ::encode(attrs, bl);
@@ -2788,12 +2821,14 @@ void ScrubMap::object::encode(bufferlist& bl) const
   ::encode(digest_present, bl);
   ::encode(nlinks, bl);
   ::encode(snapcolls, bl);
+  ::encode(omap_digest, bl);
+  ::encode(omap_digest_present, bl);
   ENCODE_FINISH(bl);
 }
 
 void ScrubMap::object::decode(bufferlist::iterator& bl)
 {
-  DECODE_START_LEGACY_COMPAT_LEN(4, 2, 2, bl);
+  DECODE_START_LEGACY_COMPAT_LEN(5, 2, 2, bl);
   ::decode(size, bl);
   ::decode(negative, bl);
   ::decode(attrs, bl);
@@ -2801,10 +2836,6 @@ void ScrubMap::object::decode(bufferlist::iterator& bl)
     ::decode(digest, bl);
     ::decode(digest_present, bl);
   }
-  else {
-    digest = 0;
-    digest_present = false;
-  }
   if (struct_v >= 4) {
     ::decode(nlinks, bl);
     ::decode(snapcolls, bl);
@@ -2813,6 +2844,10 @@ void ScrubMap::object::decode(bufferlist::iterator& bl)
      * return nlink >= 1 */
     nlinks = 0;
   }
+  if (struct_v >= 5) {
+    ::decode(omap_digest, bl);
+    ::decode(omap_digest_present, bl);
+  }
   DECODE_FINISH(bl);
 }
 
