@@ -18,8 +18,9 @@
 #include "msg/Messenger.h"
 #include "osdc/Journaler.h"
 #include "common/errno.h"
+#include "include/assert.h"
 
-#define DOUT_SUBSYS journaler
+#define dout_subsys ceph_subsys_journaler
 #undef dout_prefix
 #define dout_prefix *_dout << objecter->messenger->get_myname() << ".journaler" << (readonly ? "(ro) ":"(rw) ")
 
@@ -404,7 +405,7 @@ void Journaler::_finish_flush(int r, uint64_t start, utime_t stamp)
   if (logger) {
     utime_t lat = ceph_clock_now(cct);
     lat -= stamp;
-    logger->finc(logger_key_lat, lat);
+    logger->tinc(logger_key_lat, lat);
   }
 
   // adjust safe_pos
